==> server-core-3.6.0-sources.jar: com/xebialabs/deployit/service/importer/ImporterServiceImpl.java <==
package com.xebialabs.deployit.service.importer;

import static com.google.common.collect.Lists.newArrayList;

import java.io.File;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;

import com.google.common.collect.Lists;
import com.xebialabs.deployit.plugin.api.reflect.ReflectionsHolder;
import com.xebialabs.deployit.plugin.api.udm.ConfigurationItem;
import com.xebialabs.deployit.plugin.api.udm.Deployable;
import com.xebialabs.deployit.plugin.api.udm.artifact.SourceArtifact;
import com.xebialabs.deployit.plugin.api.validation.ValidationMessage;
import com.xebialabs.deployit.repository.RepositoryService;
import com.xebialabs.deployit.security.PermissionDeniedException;
import com.xebialabs.deployit.security.permission.Permission;
import com.xebialabs.deployit.server.api.importer.ImportSource;
import com.xebialabs.deployit.server.api.importer.ImportedPackage;
import com.xebialabs.deployit.server.api.importer.Importer;
import com.xebialabs.deployit.server.api.importer.ImportingContext;
import com.xebialabs.deployit.server.api.importer.ListableImporter;
import com.xebialabs.deployit.server.api.importer.PackageInfo;
import com.xebialabs.deployit.server.api.util.IdGenerator;
import com.xebialabs.deployit.service.replacement.MustachePlaceholderScanner;
import com.xebialabs.deployit.service.validation.Validator;

public class ImporterServiceImpl implements ImporterService {

    private RepositoryService repositoryService;

    private Validator validator;

    private File importablePackageDirectory;

	private List<Importer> importers = Lists.newArrayList();

	@Autowired
	public ImporterServiceImpl(RepositoryService repositoryService, Validator validator) {
		this.repositoryService = repositoryService;
		this.validator = validator;
	}

	@PostConstruct
	public void initImporters() {
		Set<Class<? extends Importer>> importers = ReflectionsHolder.getSubTypesOf(Importer.class);
		for (Class<? extends Importer> importer : importers) {
			try {
				if (importer.isInterface() || Modifier.isAbstract(importer.getModifiers()))
					continue;
				this.importers.add(importer.newInstance());
			} catch (Exception e) {
				throw new IllegalStateException("Could not instantiate importer: " + importer, e);
			}
		}
	}

    public void setImportablePackageDirectory(final File importablePackageDirectory) {
        this.importablePackageDirectory = importablePackageDirectory;
    }

	@Override
	public File getImportablePackageDirectory() {
		return importablePackageDirectory;
	}
	
	@Override
	public List<String> listPackages() {
		List<String> packagesFound = Lists.newArrayList();
		for (Importer importer : importers) {
			if (importer instanceof ListableImporter) {
				packagesFound.addAll(((ListableImporter) importer).list(importablePackageDirectory));
			}
		}

		Collections.sort(packagesFound);
		return packagesFound;
	}

	@Override
	public String importPackage(ImportSource source) {
		try {
			for (Importer importer : importers) {
				if (importer.canHandle(source)) {
					return doImport(source, importer);
				}
			}
			throw new ImporterException("Unable to obtain the package to import from the import source");
		} finally {
			source.cleanUp();
		}
	}

	private String doImport(ImportSource source, Importer importer) {
		ImportingContext ctx = new DefaultImportingContext();
		PackageInfo packageInfo = importer.preparePackage(source, ctx);
		boolean isUpgrade = isUpgrade(packageInfo);
		checkPermission(isUpgrade, packageInfo);
		checkImported(packageInfo);
		try {
			ImportedPackage importedPackage = importer.importEntities(packageInfo, ctx);
			scanPlaceholders(importedPackage.getDeployables());
			List<ConfigurationItem> toCreate = createEntities(importedPackage, isUpgrade);
			validate(toCreate);
			repositoryService.create(toCreate.toArray(new ConfigurationItem[toCreate.size()]));
			return importedPackage.getDeploymentPackage().getId();
		} finally {
			importer.cleanUp(packageInfo, ctx);
		}
	}

	private void scanPlaceholders(List<Deployable> deployables) {
		for (Deployable deployable : deployables) {
			if (deployable instanceof SourceArtifact) {
				((SourceArtifact) deployable).initPlaceholders(new MustachePlaceholderScanner());
			}
		}
	}

	private void checkImported(PackageInfo packageInfo) {
		if (repositoryService.checkNodeExists(IdGenerator.generateId(packageInfo.getApplicationId(), packageInfo.getApplicationVersion()))) {
			throw new ImporterException("Already imported version %s of application %s", packageInfo.getApplicationVersion(), packageInfo.getApplicationName());
		}
	}

	private List<ConfigurationItem> createEntities(ImportedPackage importedPackage, boolean isUpgrade) {
		List<ConfigurationItem> toCreate = newArrayList();
		if (!isUpgrade) {
			toCreate.add(importedPackage.getApplication());
		}
		toCreate.add(importedPackage.getDeploymentPackage());
		toCreate.addAll(importedPackage.getDeployables());
		return toCreate;
	}

	private void validate(List<ConfigurationItem> toCreate) {
		List<ValidationMessage> msgs = newArrayList();
		for (ConfigurationItem toCreateEntity : toCreate) {
			msgs.addAll(validator.validate(toCreateEntity, toCreate));
		}

		if (!msgs.isEmpty()) {
			throw new ImporterException("Import failed with the following validation errors %s", msgs.toString());
		}
	}

	private boolean isUpgrade(PackageInfo packageInfo) {
		return repositoryService.checkNodeExists(packageInfo.getApplicationId());
	}

	private void checkPermission(boolean isUpgrade, PackageInfo packageInfo) {
		if (isUpgrade) {
			checkPermission(Permission.IMPORT_UPGRADE, newArrayList(packageInfo.getApplicationId()));
		} else {
			checkPermission(Permission.IMPORT_INITIAL, Lists.<String>newArrayList());
		}
	}

	void checkPermission(Permission permission, ArrayList<String> onConfigurationItems) {
		if (!permission.getPermissionHandler().hasPermission(onConfigurationItems)) {
			throw PermissionDeniedException.forPermission(permission, onConfigurationItems);
		}
	}

}

==> server-core-3.6.3-sources.jar: com/xebialabs/deployit/service/importer/ImporterServiceImpl.java <==
package com.xebialabs.deployit.service.importer;

import static com.google.common.collect.Lists.newArrayList;

import java.io.File;
import java.lang.reflect.Modifier;
import java.util.*;

import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import com.google.common.collect.Lists;
import com.xebialabs.deployit.plugin.api.reflect.ReflectionsHolder;
import com.xebialabs.deployit.plugin.api.udm.ConfigurationItem;
import com.xebialabs.deployit.plugin.api.udm.Deployable;
import com.xebialabs.deployit.plugin.api.udm.artifact.SourceArtifact;
import com.xebialabs.deployit.plugin.api.validation.ValidationMessage;
import com.xebialabs.deployit.repository.RepositoryService;
import com.xebialabs.deployit.security.PermissionDeniedException;
import com.xebialabs.deployit.security.permission.Permission;
import com.xebialabs.deployit.server.api.importer.ImportSource;
import com.xebialabs.deployit.server.api.importer.ImportedPackage;
import com.xebialabs.deployit.server.api.importer.Importer;
import com.xebialabs.deployit.server.api.importer.ImportingContext;
import com.xebialabs.deployit.server.api.importer.ListableImporter;
import com.xebialabs.deployit.server.api.importer.PackageInfo;
import com.xebialabs.deployit.server.api.util.IdGenerator;
import com.xebialabs.deployit.service.replacement.MustachePlaceholderScanner;
import com.xebialabs.deployit.service.validation.Validator;

public class ImporterServiceImpl implements ImporterService {

    private RepositoryService repositoryService;

    private Validator validator;

    private File importablePackageDirectory;

	private List<Importer> importers = Lists.newArrayList();

	@Autowired
	public ImporterServiceImpl(RepositoryService repositoryService, Validator validator) {
		this.repositoryService = repositoryService;
		this.validator = validator;
	}

	@PostConstruct
	public void initImporters() {
		Set<Class<? extends Importer>> importers = ReflectionsHolder.getSubTypesOf(Importer.class);
		logger.debug("Found importers: {}", importers);
		for (Class<? extends Importer> importer : importers) {
			try {
				if (importer.isInterface() || Modifier.isAbstract(importer.getModifiers()))
					continue;
				logger.debug("Importer {} registered.", importer);
				this.importers.add(importer.newInstance());
			} catch (Exception e) {
				throw new IllegalStateException("Could not instantiate importer: " + importer, e);
			}
		}
		Collections.sort(this.importers, new Comparator<Object>() {
			@Override
			public int compare(Object o1, Object o2) {
				return o1.getClass().getSimpleName().compareTo(o2.getClass().getSimpleName());
			}
		});

		logger.info("Importers configured in Deployit: {}", this.importers);
	}

    public void setImportablePackageDirectory(final File importablePackageDirectory) {
        this.importablePackageDirectory = importablePackageDirectory;
    }

	@Override
	public File getImportablePackageDirectory() {
		return importablePackageDirectory;
	}
	
	@Override
	public List<String> listPackages() {
		List<String> packagesFound = Lists.newArrayList();
		for (Importer importer : importers) {
			if (importer instanceof ListableImporter) {
				packagesFound.addAll(((ListableImporter) importer).list(importablePackageDirectory));
			}
		}

		Collections.sort(packagesFound);
		return packagesFound;
	}

	@Override
	public String importPackage(ImportSource source) {
		try {
			for (Importer importer : importers) {
				if (importer.canHandle(source)) {
					return doImport(source, importer);
				}
			}
			throw new ImporterException("Unable to obtain the package to import from the import source");
		} finally {
			source.cleanUp();
		}
	}

	private String doImport(ImportSource source, Importer importer) {
		ImportingContext ctx = new DefaultImportingContext();
		PackageInfo packageInfo = importer.preparePackage(source, ctx);
		boolean isUpgrade = isUpgrade(packageInfo);
		checkPermission(isUpgrade, packageInfo);
		checkImported(packageInfo);
		try {
			ImportedPackage importedPackage = importer.importEntities(packageInfo, ctx);
			scanPlaceholders(importedPackage.getDeployables());
			List<ConfigurationItem> toCreate = createEntities(importedPackage, isUpgrade);
			validate(toCreate);
			repositoryService.create(toCreate.toArray(new ConfigurationItem[toCreate.size()]));
			return importedPackage.getDeploymentPackage().getId();
		} finally {
			importer.cleanUp(packageInfo, ctx);
		}
	}

	private void scanPlaceholders(List<Deployable> deployables) {
		for (Deployable deployable : deployables) {
			if (deployable instanceof SourceArtifact) {
				((SourceArtifact) deployable).initPlaceholders(new MustachePlaceholderScanner());
			}
		}
	}

	private void checkImported(PackageInfo packageInfo) {
		if (repositoryService.checkNodeExists(IdGenerator.generateId(packageInfo.getApplicationId(), packageInfo.getApplicationVersion()))) {
			throw new ImporterException("Already imported version %s of application %s", packageInfo.getApplicationVersion(), packageInfo.getApplicationName());
		}
	}

	private List<ConfigurationItem> createEntities(ImportedPackage importedPackage, boolean isUpgrade) {
		List<ConfigurationItem> toCreate = newArrayList();
		if (!isUpgrade) {
			toCreate.add(importedPackage.getApplication());
		}
		toCreate.add(importedPackage.getDeploymentPackage());
		toCreate.addAll(importedPackage.getDeployables());
		return toCreate;
	}

	private void validate(List<ConfigurationItem> toCreate) {
		List<ValidationMessage> msgs = newArrayList();
		for (ConfigurationItem toCreateEntity : toCreate) {
			msgs.addAll(validator.validate(toCreateEntity, toCreate));
		}

		if (!msgs.isEmpty()) {
			throw new ImporterException("Import failed with the following validation errors %s", msgs.toString());
		}
	}

	private boolean isUpgrade(PackageInfo packageInfo) {
		return repositoryService.checkNodeExists(packageInfo.getApplicationId());
	}

	private void checkPermission(boolean isUpgrade, PackageInfo packageInfo) {
		if (isUpgrade) {
			checkPermission(Permission.IMPORT_UPGRADE, newArrayList(packageInfo.getApplicationId()));
		} else {
			checkPermission(Permission.IMPORT_INITIAL, Lists.<String>newArrayList());
		}
	}

	void checkPermission(Permission permission, ArrayList<String> onConfigurationItems) {
		if (!permission.getPermissionHandler().hasPermission(onConfigurationItems)) {
			throw PermissionDeniedException.forPermission(permission, onConfigurationItems);
		}
	}

	private static final Logger logger = LoggerFactory.getLogger(ImporterServiceImpl.class);
}
